import 'dart:async';
import 'dart:convert';

import 'package:flutter_cache_manager/flutter_cache_manager.dart';
import 'package:html/dom.dart';
import 'package:html/parser.dart' show parse;
import 'package:spartathlon_app/models/BlogEntry.dart';
import 'package:spartathlon_app/util/ApiCache.dart';
import 'package:spartathlon_app/util/RaceTiming.dart';
import 'package:spartathlon_app/util/Utils.dart';

/// Provides a list of blog entries, mostly race reports
///
/// For the former, it uses a combination of parsing the blog entries on the
/// website and using the data from the JSON provided by the API.
/// This is because the API a) only returns few articles (and their storyIds are
/// not always linearly increasing, because why would they?) and because b) the
/// API doesn't return links to the image.

// TODO The API redirects to unencrypted HTTP. Talk to ISA to get that fixed.

class BlogProvider {
  // TODO Work with ISA to make HTML nicer or run own backend. These HTML hacks are awful.
  static const String baseUrl = 'https://spartathlon.gr';
  static List<BlogEntry> blogEntries;

  /// Returns a list of BlogEntries, where fullText might still be null!
  static Future<List<BlogEntry>> getBlogEntries(
      BaseCacheManager cacheManager) async {
    blogEntries = List<BlogEntry>();
    print("Getting blog entries");

    // Force refresh blog data when updates are likely
    //
    // Only refresh this overview but not the fetching of the individual items
    // Because we only care about new stories that have been added not about
    // old ones that might have changed
    if (isTwoWeeksAfterRace() && await hasConnectivity()) {
      print('Forcefully redownloading blog entries...');
      await cacheManager
          .downloadFile(baseUrl + '/en/newsroom-en/web-news-en.html');
    }
    var blogOverviewHtml = await cacheManager
        .getSingleFile(baseUrl + '/en/newsroom-en/web-news-en.html');

    String htmlContent = await blogOverviewHtml.readAsString();
    Document blogOverviewDom = parse(htmlContent);

    // Extract blog articles from DOM object using helper method
    List<Node> articles =
        _traverseDomTree(blogOverviewDom, blogOverviewDom.children[0]);

    // Handle fetching all individual blog entries concurrently
    print('Getting individual blog entries');
    List<Future<BlogEntry>> futureBlogEntries = List();
    for (Node n in articles) {
      Future<BlogEntry> fi = _getNewsItemFromArticleTag(n, cacheManager);
      futureBlogEntries.add(fi);
    }
    await Future.wait(futureBlogEntries).then((futures) {
      for (BlogEntry b in futures) {
        blogEntries.add(b);
      }
    });
    print("Finished loading blog entries");
    return blogEntries;
  }

  /// The following methods are only for internal use in this class.

  /// Returns a list of all 'article' nodes in the DOM tree
  static List<Node> _traverseDomTree(Document dom, Node start) {
    List<Node> articles = new List<Node>();
    Node node = start;
    if (node.children != null) {
      for (Node child in node.children) {
        if (child.toString() == '<html article>') {
          articles.add(child);
        }
        articles.addAll(_traverseDomTree(dom, child));
      }
    }
    return articles;
  }

  static Future<BlogEntry> _getNewsItemFromArticleTag(
      Node item, BaseCacheManager cacheManager) async {
    Node a = item.children[0];

    String url = baseUrl + a.attributes.entries.elementAt(0).value;

    int storyId;
    // Some storyIds >= 100
    try {
      storyId = int.parse(url.substring(50, 53));
    } catch (e) {
      storyId = int.parse(url.substring(50, 52));
    }

    String imageUrl =
        baseUrl + a.children[0].attributes.entries.elementAt(0).value;

    String apiRequestUrl =
        BASE_URL_API_ISA + 'getStory&storyID=' + storyId.toString();

    String headline;
    String teaserText;
    String fullText;

    var response = await cacheManager.getSingleFile(apiRequestUrl);
    String responseAsString = await response.readAsString();

    Map storyAsJson = json.decode(responseAsString)['message'] as Map;
    headline = storyAsJson['title'];
    var tmp = parse(storyAsJson['short']);
    teaserText = parse(tmp.body.text).documentElement.text;
    fullText = storyAsJson['full'];

    // Build blog entry object
    BlogEntry i = BlogEntry(
      url: url,
      storyId: storyId,
      headline: headline,
      teaserText: teaserText,
      fullText: fullText,
      imageUrl: imageUrl,
    );
    return i;
  }
}
