import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

/// Defines the app's themes: light (default) and dark.
/// We might define a black (OLED) theme in the future as well

enum AppTheme { THEME_LIGHT, THEME_DARK }

const String SETTING_THEME = 'theme';

/// Returns the ThemeData object corresponding to the given enum AppTheme
ThemeData getThemeData(AppTheme theme) {
  switch (theme) {
    case AppTheme.THEME_DARK:
      return darkTheme;
    default:
      return lightTheme;
  }
}

/// Asynchronously returns the id of the theme currently stored in the shared prefs.
/// We assume that this is consistent with the one Flutter currently uses to draw the widgets (this should be made sure elsewhere!).
Future<AppTheme> getThemeFromPrefs() async {
  SharedPreferences prefs = await SharedPreferences.getInstance();
  int themeId = prefs.getInt(SETTING_THEME);
  // Make sure theme setting exists
  while (themeId == null) {
    await setThemeIdToPrefs(AppTheme.THEME_LIGHT);
    themeId = prefs.getInt(SETTING_THEME);
  }
  return AppTheme.values[themeId];
}

/// Stores new app theme in shared preferences to persist across restarts.
/// We use an int instead of a boolean since we might want to have a black/OLED theme with value 2 in the future.
setThemeIdToPrefs(AppTheme theme) async {
  SharedPreferences prefs = await SharedPreferences.getInstance();
  print('Setting app theme to ' +
      theme.toString() +
      ' with theme ID: ' +
      theme.index.toString());
  await prefs.setInt(SETTING_THEME, theme.index);
}

bool isLight(ThemeData theme) {
  // TODO Better discriminator of themes that scales to more than two?
  return theme.brightness == Brightness.light;
}

/// Custom colors
const Color spartaBlue = Color(0xFF005596);
const Color spartaBlueDark = Color(0xFF2962FF);
const Color spartaYellowDark = Color(0xFFFFC629);

/// Light theme
final ThemeData lightTheme = ThemeData(
  brightness: Brightness.light,
  primaryColor: spartaBlue,
  primaryColorLight: Colors.white,
  accentColor: spartaBlue,
  canvasColor: Colors.grey[50],
  cardColor: Colors.white,
  dialogBackgroundColor: Colors.white,
  scaffoldBackgroundColor: Colors.grey[100],
  textTheme: Typography(platform: defaultTargetPlatform).black,
  // workaround for text themes
  textSelectionHandleColor: Colors.black54,
  toggleableActiveColor: spartaBlue,
  iconTheme: IconThemeData(color: spartaBlue),
  dividerColor: Colors.grey[400],
);

/// Dark theme
// Guideline: https://material.io/design/color/dark-theme.html
final ThemeData darkTheme = ThemeData(
  brightness: Brightness.dark,
  primaryColor: spartaBlueDark,
  primaryColorLight: spartaBlueDark,
  accentColor: spartaBlueDark,
  canvasColor: Colors.grey[850],
  cardColor: Color(0xFF242424),
  dialogBackgroundColor: Colors.grey[900],
  scaffoldBackgroundColor: Color(0xFF121212),
  textTheme: Typography(platform: defaultTargetPlatform).white,
  // workaround for text themes
  textSelectionHandleColor: Colors.grey[300],
  toggleableActiveColor: spartaYellowDark,
  iconTheme: IconThemeData(color: spartaBlueDark),
  dividerColor: Color(0x40FFFFFF),
);
