import 'package:flutter/material.dart';
import 'package:outline_material_icons/outline_material_icons.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:spartathlon_app/SpartaLocalizations.dart';
import 'package:spartathlon_app/models/Cp.dart';
import 'package:spartathlon_app/providers/CpProvider.dart';

/// Course View
///
/// Lists all checkpoints (aka CPs in Spartathlon slang)

enum CpFilterOption { All, Supporters, Live }

const String SETTING_CP_FILTER = 'cpFilter';

class CourseView extends StatefulWidget {
  CourseView({Key key}) : super(key: key);

  @override
  _CourseViewState createState() => _CourseViewState();
}

class _CourseViewState extends State<CourseView> {
  List<Cp> cps;
  List<Cp> cpsToDisplay;

  double checkpointsFilterBarHeight = 50.0;

  @override
  void initState() {
    cps = CpProvider.getCps(context);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: getCpFilterOptionFromPref(),
      builder: (BuildContext context, AsyncSnapshot<CpFilterOption> snapshot) {
        if (snapshot.hasData) {
          CpFilterOption option = snapshot.data;
          cpsToDisplay = _filterCpList(cps, option);

          return Column(
            children: [
              // CP list header bar
              Container(
                height: checkpointsFilterBarHeight,
                padding: EdgeInsets.only(left: 12.0, right: 12.0),
                decoration: BoxDecoration(
                  border: Border(
                    bottom: BorderSide(
                        color: Theme.of(context).dividerColor, width: 1.5),
                  ),
                ),
                child: Center(
                  child: Row(
                    children: [
                      Expanded(
                        child: Text(
                          SpartaLocalizations.of(context).titleCheckpoints,
                          style: TextStyle(
                            fontSize: 16.0,
                            fontWeight: FontWeight.bold,
                            color: Theme.of(context).textSelectionHandleColor,
                          ),
                        ),
                      ),
                      DropdownButtonHideUnderline(
                        child: DropdownButton(
                          value: option,
                          hint: Text(
                            SpartaLocalizations.of(context)
                                .labelButtonFilterCheckpoints,
                            style: TextStyle(
                                color:
                                    Theme.of(context).textSelectionHandleColor),
                          ),
                          items: [
                            CpFilterOption.All,
                            CpFilterOption.Supporters,
                            CpFilterOption.Live
                          ]
                              .map((CpFilterOption o) => DropdownMenuItem(
                                    value: o,
                                    child: Text(_cpFilterOptionToString(o)),
                                  ))
                              .toList(),
                          onChanged: ((CpFilterOption selection) {
                            // Store last used filter in prefs so that we can recover it
                            // when the user returns
                            setState(() {
                              cpsToDisplay = _filterCpList(cps, selection);
                            });
                            setCpFilterOptionToPrefs(selection);
                          }),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
              // CP list
              Expanded(
                child: ListView.builder(
                  itemCount: cpsToDisplay.length,
                  itemBuilder: ((BuildContext context, int position) {
                    return ExpansionTile(
                      key: ValueKey(cpsToDisplay[position].id),
                      leading: Padding(
                          padding: EdgeInsets.only(top: 4.0),
                          child: Text(
                              'CP ' + cpsToDisplay[position].id.toString())),
                      title: _getPanelTitle(cpsToDisplay[position]),
                      initiallyExpanded: false,
                      children: [
                        CpProvider.getCpExpansionTileBody(
                            cpsToDisplay[position], context)
                      ],
                    );
                  }),
                ),
              ),
            ],
          );
        } else {
          // Don't show a proper loading view because the short blue loading
          // indicator is more confusing than an empty screen
          return Container();
        }
      },
    );
  }

  Widget _getPanelTitle(Cp cp) {
    List<Widget> iconList = [];
    if (cp.firstAid) {
      iconList.add(Padding(
        padding: EdgeInsets.only(left: 4.0),
        child: Icon(
          OMIcons.localHospital,
          color: Theme.of(context).iconTheme.color,
        ),
      ));
    }
    if (cp.liveTiming) {
      iconList.add(Padding(
          padding: EdgeInsets.only(left: 4.0),
          child: Icon(
            OMIcons.insertChart,
            color: Theme.of(context).iconTheme.color,
          )));
    }
    if (cp.supporterAllowed) {
      iconList.add(Padding(
          padding: EdgeInsets.only(left: 4.0),
          child: Icon(
            OMIcons.group,
            color: Theme.of(context).iconTheme.color,
          )));
    }

    return Container(
      child: Row(
        children: [
          Expanded(
            child: Text(cp.name),
          ),
          Row(
            children: iconList,
          ),
        ],
      ),
    );
  }

  /// Filters through the list of all CPs and only adds those CPs to the display
  /// that satisfy a certain property
  /// Does NOT call setState, the caller needs to do that himself!
  List<Cp> _filterCpList(List<Cp> cps, CpFilterOption option) {
    List<Cp> newCpsToDisplay = List<Cp>();
    switch (option) {
      case CpFilterOption.Supporters:
        cps.forEach((Cp cp) {
          if (cp.supporterAllowed) {
            newCpsToDisplay.add(cp);
          }
        });
        break;
      case CpFilterOption.Live:
        cps.forEach((Cp cp) {
          if (cp.liveTiming) {
            newCpsToDisplay.add(cp);
          }
        });
        break;
      default:
        newCpsToDisplay = cps;
        break;
    }
    return newCpsToDisplay;
  }

  String _cpFilterOptionToString(CpFilterOption option) {
    switch (option) {
      case CpFilterOption.Supporters:
        return SpartaLocalizations.of(context).caseCpFilterSupporters;
      case CpFilterOption.Live:
        return SpartaLocalizations.of(context).caseCpFilterLiveTiming;
      default:
        return SpartaLocalizations.of(context).caseCpFilterAll;
    }
  }

  void setCpFilterOptionToPrefs(CpFilterOption option) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await prefs.setInt(SETTING_CP_FILTER, option.index);
  }

  Future<CpFilterOption> getCpFilterOptionFromPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    int optionIndex = prefs.getInt(SETTING_CP_FILTER);
    while (optionIndex == null) {
      await prefs.setInt(SETTING_CP_FILTER, CpFilterOption.All.index);
      optionIndex = prefs.getInt(SETTING_CP_FILTER);
    }
    return CpFilterOption.values[optionIndex];
  }
}
